@@ -7,10 +7,11 @@
 #' @param type 'kde': traditional KDE 'pdd': detrial zircon PDD
 #' @param age_range range over which to calculate density
 #' @param facet logical, facet samples?
+#' @param step specify x-axis steps
 #' @export
 #' @return  ggplot2 1d density plot with histogram
 plot_dens <- function(dat, bw=30, type='kde',
-                           age_range=c(0, 4560), facet=FALSE) {
+                           age_range=c(0, 4560), facet=FALSE, step=200) {
   if (facet) {
     l <- lapply(split(dat, factor(dat$sample)), calc_dens, bw=bw,
                 type=type, age_range=age_range)
@@ -20,7 +21,7 @@ plot_dens <- function(dat, bw=30, type='kde',
                                         ggplot2::aes_string(x='x', y='y'))
     gplot <- gplot + ggplot2::facet_grid(sample ~ ., scale='free_y')
     gplot <- gplot + plot_labels(ylab = 'Density') + plot_bw_theme() +
-      plot_axis_lim(xlim=age_range)
+      plot_axis_lim(xlim=age_range, step=step)
     return(gplot)
   } else {
     dens <- calc_dens(dat, bw=bw, type=type,
@@ -29,7 +30,7 @@ plot_dens <- function(dat, bw=30, type='kde',
     gplot <- gplot + ggplot2::geom_path(data=dens,
                                         ggplot2::aes_string(x='x', y='y'))
     gplot <- gplot + plot_labels(ylab = 'Density') + plot_bw_theme() +
-      plot_axis_lim(xlim=age_range)
+      plot_axis_lim(xlim=age_range, step=step)
     return(gplot)
   }
 }
@@ -44,10 +45,11 @@ plot_dens <- function(dat, bw=30, type='kde',
 #' @param type 'kde': traditional KDE 'pdd': detrial zircon PDD
 #' @param age_range range over which to calculate density
 #' @param facet logical, facet samples?
+#' @param step specify x-axis steps
 #' @export
 #' @return  ggplot2 1d density plot with histogram
 plot_dens_hist <- function(dat, bw=30, binwidth=50, type='kde',
-                              age_range=c(0, 4560), facet=FALSE) {
+                              age_range=c(0, 4560), facet=FALSE, step=200) {
   if (facet) {
     l <- lapply(split(dat, factor(dat$sample)), calc_dens_hist, bw=bw,
                 binwidth=binwidth, type=type, age_range=age_range)
@@ -63,7 +65,7 @@ plot_dens_hist <- function(dat, bw=30, binwidth=50, type='kde',
     gplot <- gplot + ggplot2::geom_path(data=dens,
                                         ggplot2::aes_string(x='x', y='y'))
     gplot <- gplot + plot_labels(ylab = 'Count') + plot_bw_theme() +
-      plot_axis_lim(xlim=age_range)
+      plot_axis_lim(xlim=age_range, step=step)
     return(gplot)
     } else {
       dens <- calc_dens_hist(dat, binwidth=binwidth, bw=bw, type=type,
@@ -78,7 +80,7 @@ plot_dens_hist <- function(dat, bw=30, binwidth=50, type='kde',
       gplot <- gplot + ggplot2::geom_path(data=dens,
                                           ggplot2::aes_string(x='x', y='y'))
       gplot <- gplot + plot_labels(ylab = 'Count') + plot_bw_theme() +
-        plot_axis_lim(xlim=age_range)
+        plot_axis_lim(xlim=age_range, step=step)
       return(gplot)
   }
 }
@@ -103,12 +105,39 @@ plot_labels <- function(xlab = 'Age (Ma)', ylab = 'Density') {
 #'
 #' @param xlim x-axis limit
 #' @param ylim y-axis limit
-#'
+#' @param step specify x-axis steps
 #' @return list of ggplot2::coord_cartesian object
 #' @export
 #'
-plot_axis_lim <- function(xlim = c(0, 4560), ylim=NULL) {
-  p_x_lim <- list(ggplot2::coord_cartesian(xlim=xlim, ylim=ylim))
+plot_axis_lim <- function(xlim = c(0, 4560), step=200, ylim=NULL) {
+  p_x_lim <- list(ggplot2::scale_x_continuous(limits=xlim,
+                                           breaks=seq(xlim[1], xlim[2], step),
+                                           expand=c(0, 0)
+                                           ),
+                  ggplot2::scale_y_continuous(limits=ylim)
+  )
+}
+
+#' Modify text options of plots
+#'
+#' @param font_name Name of font to use
+#' @param title_size Font size of x- and y-axis titles
+#' @param label_size Font size of x- and y-axis tick labels
+#' @param legend_size Font size of legend
+#' @param strip_text_y_size Font size of vertical panel text
+#'
+#' @export
+#'
+plot_text_options <- function(font_name = 'Helvetica', title_size = 10,
+                              label_size = 7, legend_size = 10,
+                              strip_text_y_size = 8) {
+  text_options <- list(ggplot2::theme(
+    text=ggplot2::element_text(family=font_name),
+    axis.title=ggplot2::element_text(size=title_size),
+    axis.text=ggplot2::element_text(size=label_size),
+    legend.text=ggplot2::element_text(size=legend_size),
+    legend.title=ggplot2::element_blank(),
+    strip.text.y=ggplot2::element_text(size=strip_text_y_size)))
 }
 
 #' Stripped down theme for ggplot2
@@ -136,6 +165,9 @@ plot_bw_theme <- function() {
 #'
 plot_ecdf <- function(dat, mult_ecdf=FALSE, column='age', conf=FALSE,
                       guide=TRUE, alpha=0.05) {
+  if (column == 't_dm2') {
+    dat <- dat[!is.na(dat$ehf_i), ]
+  }
   if (mult_ecdf) {
     l <- lapply(split(dat, factor(dat$sample)), calc_dkw, column=column,
                 alpha=alpha)
@@ -227,10 +259,11 @@ plot_hf <- function(dat, range=c(0, 4560), plot_type='ehf', guide=TRUE,
     gplot <- gplot +
       ggplot2::geom_point(data=dat,
                           ggplot2::aes_string(x='age', y='ehf_i',
-                                              fill='sample'),
-                          shape=21,
+                                              fill='sample',
+                                              shape='sample'),
                           color='black',
                           size=3)
+    gplot <- gplot + plot_point_scale()
     gplot <- gplot + ggplot2::guides(linetype=FALSE)
   } else {
     if (plot_type == 'hfhf') {
@@ -268,13 +301,15 @@ plot_hf <- function(dat, range=c(0, 4560), plot_type='ehf', guide=TRUE,
         ggplot2::geom_point(data=dat,
                             ggplot2::aes_string(x='age',
                                                 y='hf_i',
-                                                fill='sample'),
-                            color='black', shape=21, size=3)
+                                                fill='sample',
+                                                shape='sample'),
+                            color='black', size=3)
+      gplot <- gplot + plot_point_scale()
       gplot <- gplot + ggplot2::guides(linetype=FALSE)
       }
   }
   if (guide == FALSE) {
-    gplot <- gplot + ggplot2::guides(fill=FALSE)
+    gplot <- gplot + ggplot2::guides(fill=FALSE, shape=FALSE)
   }
   gplot
 }
@@ -295,7 +330,7 @@ plot_hf <- function(dat, range=c(0, 4560), plot_type='ehf', guide=TRUE,
 plot_quantiles <- function(dat, column='t_dm2', conf=FALSE, alpha=0.05, type=8,
                            guide=TRUE, mix=FALSE, mix_data=NULL) {
   quants <- calc_quantiles(dat=dat, column=column, alpha=alpha, type=type)
-  line <- data.frame(x=c(0, 4560), y=c(0, 4560))
+  line <- data.frame(x=seq(0, 4560), y=seq(0, 4560))
   gplot <- ggplot2::ggplot() +
     ggplot2::geom_line(data=line, ggplot2::aes_string(x='x', y='y'))
   if (mix & !is.null(mix_data)) {
@@ -322,14 +357,47 @@ plot_quantiles <- function(dat, column='t_dm2', conf=FALSE, alpha=0.05, type=8,
   gplot <- gplot +
     ggplot2::geom_point(data=quants, ggplot2::aes_string(x='twentyfive',
                                                          y='seventyfive',
-                                                         fill='sample'),
+                                                         fill='sample',
+                                                         shape='sample'),
                         color='black',
-                        shape=21,
                         size=3)
   gplot <- gplot + plot_bw_theme() + plot_labels(xlab='Lower quartile (Ma)',
                                                  ylab='Upper quartile (Ma)')
+    gplot <- gplot + plot_point_scale()
   if (guide == FALSE) {
-    gplot <- gplot + ggplot2::guides(fill=FALSE)
+    gplot <- gplot + ggplot2::guides(fill=FALSE, shape=FALSE)
   }
   gplot
 }
+
+#' Add manual shape scale to scatter plot
+#'
+#' @export
+#'
+plot_point_scale <- function() {
+  point_scale <- list(ggplot2::scale_shape_manual(values=rep(c(21, 22, 23, 24,
+                                                               25), 5)))
+}
+
+#' Tile plot of 1-O matrix
+#'
+#' @param dat data.frame
+#' @param type What to plot
+#'
+#' @export
+#'
+plot_tile <- function(dat, type) {
+  tiles <- make_tiling(dat, type=type)
+  gplot <- ggplot2::ggplot(data=tiles) +
+    ggplot2::geom_tile(ggplot2::aes_string(x='x', y='y', fill='z'),
+                       color='black') +
+    plot_labels(xlab='', ylab='') +
+    ggplot2::guides(fill=FALSE) +
+    ggplot2::scale_fill_manual(values=c('#1a9641', 'white', '#d7191c'),
+                      na.value='grey80') +
+    plot_bw_theme() + ggplot2::theme(axis.text.x =
+                                       ggplot2::element_text(angle = 90,
+                                                             hjust = 1)) +
+    ggplot2::scale_x_discrete(expand=c(0, 0)) +
+    ggplot2::scale_y_discrete(expand=c(0, 0))
+}
